#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include "bootcode.h"

typedef unsigned long u32;

#define SECTOR_SIZE	512

unsigned char old[SECTOR_SIZE], new[SECTOR_SIZE];

main(int argc, char **argv) {
    int fd, len, lread;

    if (argc < 2 || argc > 3 || !strcmp(argv[1], "-h")) {
	char *p;
	if ((p = strrchr(*argv, '/')) != NULL)
	    p++;
	else
	    p = *argv;
	fprintf(stderr,
		"\nUsage: %s part_name [ offset ]\n"
		"  - <part_name> is the device which contains the boot sector "
		"to be patched ;\n"
		"    it is mandatory, but can be a file. eg: /dev/sdb1.\n"
		"  - <offset> is the partition offset within the physical "
		"disk, in sectors.\n"
		"    this is used by the boot sector to know from where to "
		"get the data.\n"
		"    This value is reported in the 'Start' column for the "
		"line <part_name> by\n"
		"    the following command : sfdisk -uS -l <disk_name>\n"
		"    If unspecified, will be left unchanged."
		"\n\n"
		,p);
	exit(1);
    }

    fd = open(argv[1], O_RDWR);
    if (fd < 0) {
	perror("open(device)");
	exit(2);
    }

    lread = 0;
    while (lread < SECTOR_SIZE) {
	if ((len = read(fd, old + lread, SECTOR_SIZE - lread)) <= 0) {
	    perror("read(device)");
	    exit(3);
	}
	lread += len;
    }

    if (old[0] != 0xEB && old[0] != 0xE9 ||
	old[510] != 0x55 || old[511] != 0xAA) {
	fprintf(stderr, "<%s> does not contain a valid boot sector.\n"
		"It should start with 0xE9 or 0xEB and end with 0x55 0xAA\n",
		argv[1]);
	exit(4);
    }

    /* create new sector, but keep old geometry */
    memcpy(new, bootcode, SECTOR_SIZE);
    memcpy(new + 0x0B, old + 0x0B, 0x3E - 0x0B);

    /* force the boot sector to automatically detect the drive number */
    new[0x24] = 0xff;

    if (argc > 2) {
	u32 start, old_start;

	old_start = old[0x1C] + (old[0x1D]<<8) + (old[0x1E]<<16) + (old[0x1F]<<24);
	start = atol(argv[2]);

	fprintf(stderr,
		"Changed boot sector offset from %d(0x%08x) to %d(0x%08x)\n",
		old_start, old_start, start, start);

	new[0x1C] = start;
	new[0x1D] = start >> 8;
	new[0x1E] = start >> 16;
	new[0x1F] = start >> 24;
    }	       

    lseek(fd, 0, SEEK_SET);
    if (write(fd, new, SECTOR_SIZE) != SECTOR_SIZE) {
	perror("write(device)");
	/* try to restore old data before exiting */
	write(fd, old, SECTOR_SIZE);
	exit(5);
    }
    fprintf(stderr, "Done !\n");
    exit (0);
}
